@@ -4,8 +4,8 @@ use 5.010;
 use strict;
 use warnings;
 
-use Exporter;
-use Scalar::Util qw( looks_like_number );;
+use parent 'Exporter';
+use Scalar::Util qw( looks_like_number );
 
 use vars qw( $VERSION @ISA @EXPORT );
 
@@ -15,57 +15,64 @@ Carp::Assert::More - Convenience assertions for common situations
 
 =head1 VERSION
 
-Version 2.4.0
+Version 2.9.0
 
 =cut
 
-BEGIN {
-    $VERSION = '2.4.0';
-    @ISA = qw(Exporter);
-    @EXPORT = qw(
-        assert_all_keys_in
-        assert_aoh
-        assert_arrayref
-        assert_arrayref_nonempty
-        assert_arrayref_of
-        assert_arrayref_all
-        assert_cmp
-        assert_coderef
-        assert_context_nonvoid
-        assert_context_scalar
-        assert_datetime
-        assert_defined
-        assert_empty
-        assert_exists
-        assert_fail
-        assert_hashref
-        assert_hashref_nonempty
-        assert_in
-        assert_integer
-        assert_is
-        assert_isa
-        assert_isa_in
-        assert_isnt
-        assert_keys_are
-        assert_lacks
-        assert_like
-        assert_listref
-        assert_negative
-        assert_negative_integer
-        assert_nonblank
-        assert_nonempty
-        assert_nonnegative
-        assert_nonnegative_integer
-        assert_nonref
-        assert_nonzero
-        assert_nonzero_integer
-        assert_numeric
-        assert_positive
-        assert_positive_integer
-        assert_undefined
-        assert_unlike
-    );
-}
+our $VERSION = '2.9.0';
+our @EXPORT = qw(
+    assert
+    assert_all_keys_in
+    assert_and
+    assert_aoh
+    assert_arrayref
+    assert_arrayref_nonempty
+    assert_arrayref_nonempty_of
+    assert_arrayref_of
+    assert_arrayref_all
+    assert_cmp
+    assert_coderef
+    assert_context_list
+    assert_context_nonvoid
+    assert_context_scalar
+    assert_context_void
+    assert_datetime
+    assert_defined
+    assert_empty
+    assert_exists
+    assert_fail
+    assert_hashref
+    assert_hashref_nonempty
+    assert_in
+    assert_integer
+    assert_integer_between
+    assert_is
+    assert_isa
+    assert_isa_in
+    assert_isnt
+    assert_keys_are
+    assert_lacks
+    assert_like
+    assert_listref
+    assert_negative
+    assert_negative_integer
+    assert_nonblank
+    assert_nonempty
+    assert_nonnegative
+    assert_nonnegative_integer
+    assert_nonref
+    assert_nonzero
+    assert_nonzero_integer
+    assert_numeric
+    assert_numeric_between
+    assert_or
+    assert_positive
+    assert_positive_integer
+    assert_regex
+    assert_undefined
+    assert_unlike
+    assert_xor
+);
 
 my $INTEGER = qr/^-?\d+$/;
 
@@ -100,6 +107,24 @@ have no excuse to not use them.
 
 =head1 SIMPLE ASSERTIONS
 
+=head2 assert( $condition [, $name] )
+
+Asserts that C<$condition> is a true value.  This is the same as C<assert>
+in C<Carp::Assert>, provided as a convenience.
+
+=cut
+
+sub assert($;$) {
+    my $condition = shift;
+    my $name = shift;
+
+    return if $condition;
+
+    require Carp;
+    &Carp::confess( _failure_msg($name) );
+}
+
+
 =head2 assert_is( $string, $match [,$name] )
 
 Asserts that I<$string> is the same string value as I<$match>.
@@ -362,6 +387,84 @@ sub assert_nonblank($;$) {
 }
 
 
+=head1 BOOLEAN ASSERTIONS
+
+These boolean assertions help make diagnostics more useful.
+
+If you use C<assert> with a boolean condition:
+
+    assert( $x && $y, 'Both X and Y should be true' );
+
+you can't tell why it failed:
+
+    Assertion (Both X and Y should be true) failed!
+     at .../Carp/Assert/More.pm line 123
+            Carp::Assert::More::assert(undef, 'Both X and Y should be true') called at foo.pl line 16
+
+But if you use C<assert_and>:
+
+    assert_and( $x, $y, 'Both X and Y should be true' );
+
+the stacktrace tells you which half of the expression failed.
+
+    Assertion (Both X and Y should be true) failed!
+     at .../Carp/Assert/More.pm line 123
+            Carp::Assert::More::assert_and('thing', undef, 'Both X and Y should be true') called at foo.pl line 16
+
+=head2 assert_and( $x, $y [, $name] )
+
+Asserts that both C<$x> and C<$y> are true.
+
+=cut
+
+sub assert_and($$;$) {
+    my $x    = shift;
+    my $y    = shift;
+    my $name = shift;
+
+    return if $x && $y;
+
+    require Carp;
+    &Carp::confess( _failure_msg($name) );
+}
+
+
+=head2 assert_or( $x, $y [, $name] )
+
+Asserts that at least one of C<$x> or C<$y> are true.
+
+=cut
+
+sub assert_or($$;$) {
+    my $x    = shift;
+    my $y    = shift;
+    my $name = shift;
+
+    return if $x || $y;
+
+    require Carp;
+    &Carp::confess( _failure_msg($name) );
+}
+
+=head2 assert_xor( $x, $y [, $name] )
+
+Asserts that C<$x> is true, or C<$y> is true, but not both.
+
+=cut
+
+sub assert_xor($$;$) {
+    my $x    = shift;
+    my $y    = shift;
+    my $name = shift;
+
+    return if $x && !$y;
+    return if $y && !$x;
+
+    require Carp;
+    &Carp::confess( _failure_msg($name) );
+}
+
+
 =head1 NUMERIC ASSERTIONS
 
 =head2 assert_numeric( $n [, $name] )
@@ -592,6 +695,58 @@ sub assert_negative_integer($;$) {
 }
 
 
+=head2 assert_numeric_between( $n, $lo, $hi [, $name ] )
+
+Asserts that the value of I<$this> is defined, numeric and between C<$lo>
+and C<$hi>, inclusive.
+
+    assert_numeric_between( 15, 10, 100 );  # pass
+    assert_numeric_between( 10, 15, 100 );  # FAIL
+    assert_numeric_between( 3.14, 1, 10 );  # pass
+
+=cut
+
+sub assert_numeric_between($$$;$) {
+    my $n    = shift;
+    my $lo   = shift;
+    my $hi   = shift;
+    my $name = shift;
+
+    if ( Scalar::Util::looks_like_number( $n ) ) {
+        return if $lo <= $n && $n <= $hi;
+    }
+
+    require Carp;
+    &Carp::confess( _failure_msg($name) );
+}
+
+
+=head2 assert_integer_between( $n, $lo, $hi [, $name ] )
+
+Asserts that the value of I<$this> is defined, an integer, and between C<$lo>
+and C<$hi>, inclusive.
+
+    assert_integer_between( 15, 10, 100 );  # pass
+    assert_integer_between( 10, 15, 100 );  # FAIL
+    assert_integer_between( 3.14, 1, 10 );  # FAIL
+
+=cut
+
+sub assert_integer_between($$$;$) {
+    my $n    = shift;
+    my $lo   = shift;
+    my $hi   = shift;
+    my $name = shift;
+
+    if ( defined($n) && $n =~ $INTEGER ) {
+        return if $lo <= $n && $n <= $hi;
+    }
+
+    require Carp;
+    &Carp::confess( _failure_msg($name) );
+}
+
+
 =head1 REFERENCE ASSERTIONS
 
 =head2 assert_isa( $this, $type [, $name ] )
@@ -869,8 +1024,8 @@ sub assert_arrayref_nonempty($;$) {
 
 =head2 assert_arrayref_of( $ref, $type [, $name] )
 
-Asserts that I<$ref> is reference to an array that has at least one
-element in it, and every one of those elements is of type I<$type>.
+Asserts that I<$ref> is reference to an array, and any/all elements are
+of type I<$type>.
 
 For example:
 
@@ -887,6 +1042,44 @@ sub assert_arrayref_of($$;$) {
     my $ok;
     my @why;
 
+    if ( ref($ref) eq 'ARRAY' || (Scalar::Util::blessed( $ref ) && $ref->isa( 'ARRAY' )) ) {
+        my $n = 0;
+        for my $i ( @{$ref} ) {
+            if ( !( ( Scalar::Util::blessed( $i ) && $i->isa( $type ) ) || (ref($i) eq $type) ) ) {
+                push @why, "Element #$n is not of type $type";
+            }
+            ++$n;
+        }
+        $ok = !@why;
+    }
+
+    if ( !$ok ) {
+        require Carp;
+        &Carp::confess( _failure_msg($name), @why );
+    }
+
+    return;
+}
+
+
+=head2 assert_arrayref_nonempty_of( $ref, $type [, $name] )
+
+Asserts that I<$ref> is reference to an array, that it has at least one
+element, and that all elements are of type I<$type>.
+
+This is the same function as C<assert_arrayref_of>, except that it also
+requires at least one element.
+
+=cut
+
+sub assert_arrayref_nonempty_of($$;$) {
+    my $ref  = shift;
+    my $type = shift;
+    my $name = shift;
+
+    my $ok;
+    my @why;
+
     if ( ref($ref) eq 'ARRAY' || (Scalar::Util::blessed( $ref ) && $ref->isa( 'ARRAY' )) ) {
         if ( scalar @{$ref} > 0 ) {
             my $n = 0;
@@ -1012,6 +1205,27 @@ sub assert_coderef($;$) {
 }
 
 
+=head2 assert_regex( $ref [,$name] )
+
+Asserts that I<$ref> is defined, and is a reference to a regex.
+
+It is functionally the same as C<assert_isa( $ref, 'Regexp' )>.
+
+=cut
+
+sub assert_regex($;$) {
+    my $ref = shift;
+    my $name = shift;
+
+    if ( ref($ref) eq 'Regexp' ) {
+        return;
+    }
+
+    require Carp;
+    &Carp::confess( _failure_msg($name) );
+}
+
+
 =head1 TYPE-SPECIFIC ASSERTIONS
 
 =head2 assert_datetime( $date )
@@ -1299,7 +1513,7 @@ Given this function:
     sub something {
         ...
 
-        assert_context_scalar();
+        assert_context_nonvoid();
 
         return $important_value;
     }
@@ -1319,11 +1533,53 @@ must not be called in void context" is provided.
 =cut
 
 sub assert_context_nonvoid(;$) {
-    my @caller = caller(1);
+    my (undef, undef, undef, $subroutine, undef, $wantarray) = caller(1);
 
-    return if defined($caller[5]);
+    return if defined($wantarray);
 
-    my $name = shift // "$caller[3] must not be called in void context";
+    my $name = $_[0] // "$subroutine must not be called in void context";
+
+    require Carp;
+    &Carp::confess( _failure_msg($name) );
+}
+
+
+=head2 assert_context_void( [$name] )
+
+Verifies that the function currently being executed has been called
+in void context.  This is for functions that do not return anything
+meaningful.
+
+Given this function:
+
+    sub something {
+        ...
+
+        assert_context_void();
+
+        return; # No meaningful value.
+    }
+
+These calls to C<something> will fail:
+
+    my $val = something();
+    my @things = something();
+
+but this will pass:
+
+    something();
+
+If the C<$name> argument is not passed, a default message of "<funcname>
+must be called in void context" is provided.
+
+=cut
+
+sub assert_context_void(;$) {
+    my (undef, undef, undef, $subroutine, undef, $wantarray) = caller(1);
+
+    return if not defined($wantarray);
+
+    my $name = $_[0] // "$subroutine must be called in void context";
 
     require Carp;
     &Carp::confess( _failure_msg($name) );
@@ -1361,12 +1617,52 @@ must be called in scalar context" is provided.
 =cut
 
 sub assert_context_scalar(;$) {
-    my @caller = caller(1);
-    my $wantarray = $caller[5];
+    my (undef, undef, undef, $subroutine, undef, $wantarray) = caller(1);
 
     return if defined($wantarray) && !$wantarray;
 
-    my $name = shift // "$caller[3] must be called in scalar context";
+    my $name = $_[0] // "$subroutine must be called in scalar context";
+
+    require Carp;
+    &Carp::confess( _failure_msg($name) );
+}
+
+
+=head2 assert_context_list( [$name] )
+
+Verifies that the function currently being executed has been called in
+list context.
+
+Given this function:
+
+    sub something {
+        ...
+
+        assert_context_scalar();
+
+        return @values;
+    }
+
+This call to C<something> will pass:
+
+    my @vals = something();
+
+but these will fail:
+
+    something();
+    my $thing = something();
+
+If the C<$name> argument is not passed, a default message of "<funcname>
+must be called in list context" is provided.
+
+=cut
+
+sub assert_context_list(;$) {
+    my (undef, undef, undef, $subroutine, undef, $wantarray) = caller(1);
+
+    return if $wantarray;
+
+    my $name = shift // "$subroutine must be called in list context";
 
     require Carp;
     &Carp::confess( _failure_msg($name) );
@@ -1404,7 +1700,7 @@ sub _failure_msg {
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2005-2023 Andy Lester
+Copyright 2005-2025 Andy Lester
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the Artistic License version 2.0.
